@@ -2,7 +2,7 @@
  * wiggle - apply rejected patches
  *
  * Copyright (C) 2003 Neil Brown <neilb@cse.unsw.edu.au>
- * Copyright (C) 2010 Neil Brown <neilb@suse.de>
+ * Copyright (C) 2010-2013 Neil Brown <neilb@suse.de>
  *
  *
  *    This program is free software; you can redistribute it and/or modify
@@ -16,8 +16,7 @@
  *    GNU General Public License for more details.
  *
  *    You should have received a copy of the GNU General Public License
- *    along with this program; if not, write to the Free Software Foundation, Inc.,
- *    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *    along with this program.
  *
  *    Author: Neil Brown
  *    Email: <neilb@suse.de>
@@ -67,12 +66,14 @@ int split_patch(struct stream f, struct stream *f1, struct stream *f2)
 	int a, b, c, d;
 	int lineno = 0;
 	char before[100], after[100];
+	char func[100];
 
 	f1->body = f2->body = NULL;
 
 	r1.body = xmalloc(f.len);
 	r2.body = xmalloc(f.len);
 	r1.len = r2.len = 0;
+	func[0] = 0;
 
 	cp = f.body;
 	end = f.body+f.len;
@@ -86,7 +87,7 @@ int split_patch(struct stream f, struct stream *f1, struct stream *f2)
 		lineno++;
 		switch (state) {
 		case 0:
-			if (sscanf(cp, "@@ -%s +%s @@", before, after) == 2) {
+			if (sscanf(cp, "@@ -%99s +%99s @@%99[^\n]", before, after, func) >= 2) {
 				int ok = 1;
 				if (sscanf(before, "%d,%d", &a, &b) == 2)
 					acnt = b;
@@ -111,15 +112,28 @@ int split_patch(struct stream f, struct stream *f1, struct stream *f2)
 			} else if (sscanf(cp, "--- %d,%d ----", &c, &d) == 2) {
 				bcnt = d-c+1;
 				state = 2;
-			}
+			} else
+				sscanf(cp, "***************%99[^\n]", func);
+
 			skip_eol(&cp, end);
 			if (state == 1 || state == 3) {
+				char *f;
 				char buf[20];
 				buf[0] = 0;
 				chunks++;
-				sprintf(buf+1, "%5d %5d %5d\n", chunks, a, acnt);
-				memcpy(r1.body+r1.len, buf, 20);
-				r1.len += 20;
+				sprintf(buf+1, "%5d %5d %5d", chunks, a, acnt);
+				memcpy(r1.body+r1.len, buf, 18);
+				r1.len += 18;
+				f = func;
+				while (*f == ' ')
+					f++;
+				if (*f) {
+					r1.body[r1.len++] = ' ';
+					strcpy(r1.body + r1.len, f);
+					r1.len += strlen(f);
+				}
+				r1.body[r1.len++] = '\n';
+				r1.body[r1.len++] = '\0';
 			}
 			if (state == 2 || state == 3) {
 				char buf[20];
@@ -128,6 +142,8 @@ int split_patch(struct stream f, struct stream *f1, struct stream *f2)
 				memcpy(r2.body+r2.len, buf, 20);
 				r2.len += 20;
 			}
+			if (state)
+				func[0] = 0;
 			break;
 		case 1:
 			if ((*cp == ' ' || *cp == '!' || *cp == '-' || *cp == '+')
@@ -195,7 +211,6 @@ int split_patch(struct stream f, struct stream *f1, struct stream *f2)
  */
 int split_merge(struct stream f, struct stream *f1, struct stream *f2, struct stream *f3)
 {
-	int lineno;
 	int state = 0;
 	char *cp, *end;
 	struct stream r1, r2, r3;
@@ -219,7 +234,6 @@ int split_merge(struct stream f, struct stream *f1, struct stream *f2, struct st
 		 *  5 in file 1/3
 		 */
 		int len = end-cp;
-		lineno++;
 		switch (state) {
 		case 0:
 			if (len >= 8 &&
@@ -318,6 +332,8 @@ int split_merge(struct stream f, struct stream *f1, struct stream *f2, struct st
 			break;
 		}
 	}
+	if (cp > end)
+		abort();
 	*f1 = r1;
 	*f2 = r2;
 	*f3 = r3;
